from typing import AsyncGenerator, Optional, Any
import re

from sqlalchemy.ext.asyncio import (
    AsyncEngine,
    AsyncSession,
    async_sessionmaker,
    create_async_engine,
)
from sqlalchemy import create_engine, Engine
from sqlalchemy.pool import NullPool
from sqlmodel import SQLModel

from fastappear.config import settings
from fastappear.utils.logger import logger

log = logger(__name__)


def _make_async_uri(db_uri: str) -> str:
    """
    Ensure the DB URI uses an async driver suitable for SQLAlchemy async engines.
    If the incoming URI is like 'postgresql://...', convert to 'postgresql+asyncpg://...'.
    If it already contains '+', assume it's correct.
    """
    if "+asyncpg" in db_uri or "+psycopg" in db_uri:
        return db_uri

    if db_uri.startswith("postgresql://"):
        return db_uri.replace("postgresql://", "postgresql+asyncpg://", 1)

    return db_uri


_async_db_uri = _make_async_uri(settings.db_uri)
log.debug("Async DB URI: %s", re.sub(r"://.*?:.*?@", "://<redacted>@", _async_db_uri))

async_engine: AsyncEngine = create_async_engine(
    _async_db_uri,
    echo=False,
    pool_pre_ping=True,
)

async_session: async_sessionmaker[AsyncSession] = async_sessionmaker(
    bind=async_engine,
    expire_on_commit=False,
    class_=AsyncSession,
)


async def get_db() -> AsyncGenerator[AsyncSession, None]:
    """
    FastAPI dependency — yields an AsyncSession.
    Example:
        @app.get("/items/")
        async def read_items(db: AsyncSession = Depends(get_db)):
            result = await db.execute(select(Item))
    """
    async with async_session() as session:
        try:
            yield session
        except Exception:
            await session.rollback()
            raise
        finally:
            await session.close()


async def init_models(engine: Optional[AsyncEngine] = None) -> None:
    """
    Create tables for all SQLModel models.
    Call at app startup if you want SQLAlchemy to create tables automatically.
    """
    eng = engine or async_engine
    async with eng.begin() as conn:
        # Use checkfirst=True to avoid "table already exists" errors
        def create_tables(connection: Any) -> None:
            SQLModel.metadata.create_all(connection, checkfirst=True)

        await conn.run_sync(create_tables)
    log.info("Database tables created (if they did not exist).")


def get_sync_engine(db_uri: Optional[str] = None) -> Engine:
    """
    Return a synchronous SQLAlchemy Engine. Useful for Alembic or scripts that are sync-only.
    Converts a +asyncpg URI back to a sync form if necessary.
    """
    uri = db_uri or settings.db_uri
    if "+asyncpg" in uri:
        sync_uri = uri.replace("+asyncpg", "")
    else:
        sync_uri = uri

    return create_engine(sync_uri, poolclass=NullPool)



import time
from typing import Any, Callable, TypeVar, Dict, AsyncGenerator
from contextlib import asynccontextmanager
import uvicorn
from fastapi import FastAPI, Request, Response, APIRouter
from fastapi.middleware.cors import CORSMiddleware
from src.config import settings
from src.utils.db import init_models, async_session, async_engine
from sqlalchemy.ext.asyncio import AsyncSession
from src.utils.logger import logger

description = """
<Application_Name> API's
"""

log = logger()


async def create_default_admin_if_missing(session: AsyncSession) -> None:
    """No-op helper to create a default admin if it does not exist.

    This is intentionally lightweight — it logs the operation. Replace with your
    production logic (creating a seeded admin user) in a real application.
    """
    log.debug("checked for default admin (no-op)")
    return None


@asynccontextmanager
async def lifespan(app: FastAPI) -> AsyncGenerator[None, None]:
    """Application lifespan context.

    - initializes the DB models at startup
    - tries to create a default admin if missing (no-op in minimal setup)
    - disposes the async engine at shutdown
    """
    log.info("Starting <Application_Name> API...")
    await init_models()
    # create_default_admin_if_missing is optional; implement as a no-op if not available
    try:
        async with async_session() as session:
            await create_default_admin_if_missing(session)
    except NameError:
        log.debug("create_default_admin_if_missing not available, skipping")
    except Exception:
        log.exception("Failed to create default admin, continuing startup")

    log.info("Startup complete.")

    yield

    log.info("Shutting down <Application_Name> API...")
    # Properly dispose the underlying engine when shutting down
    try:
        await async_engine.dispose()
    except Exception:
        log.exception("Error on engine dispose")
    log.info("Shutdown complete.")


app = FastAPI(
    title="<Application_Name>",
    description=description,
    version="1.0.0",
    docs_url="/docs",
    redoc_url="/redoc",
    root_path=settings.root_path,
    lifespan=lifespan,
)

app.add_middleware(
    CORSMiddleware,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
    allow_origins=["*"],
)


@app.get("/", tags=["Health"])
async def health_check() -> Dict[str, str]:
    return {"status": "ok", "message": "<Application_Name> API is running"}


F = TypeVar("F", bound=Callable[..., Any])


@app.middleware("http")
async def process_time_log_middleware(
    request: Request, call_next: Callable[[Request], Any]
) -> Response:
    start_time = time.time()
    response: Response = await call_next(request)
    process_time = str(round(time.time() - start_time, 3))
    response.headers["X-Process-Time"] = process_time
    log.info(
        "Method=%s Path=%s StatusCode=%s ProcessTime=%s",
        request.method,
        request.url.path,
        response.status_code,
        process_time,
    )
    return response


## Minimal routers for demonstration
login_router = APIRouter(prefix="/auth", tags=["auth"])
register_router = APIRouter(prefix="/auth", tags=["auth"])


@login_router.post("/login")
async def _login() -> Dict[str, str]:
    return {"detail": "login endpoint (stub)"}


@register_router.post("/register")
async def _register() -> Dict[str, str]:
    return {"detail": "register endpoint (stub)"}


app.include_router(login_router)
app.include_router(register_router)

if __name__ == "__main__":
    # When running locally, pass the app object directly to uvicorn.run
    uvicorn.run(
        app,
        host="0.0.0.0",
        port=8000,
        log_level="debug",
        reload=True,
    )



import logging
import sys

from src.config import settings

# ANSI color codes
COLORS = {
    "DEBUG": "\033[36m",  # Cyan
    "INFO": "\033[32m",  # Green
    "WARNING": "\033[33m",  # Yellow
    "ERROR": "\033[31m",  # Red
    "CRITICAL": "\033[35m",  # Magenta
}
RESET = "\033[0m"

LOG_FORMAT = "[%(levelname)s] %(asctime)s - %(message)s"
DATEFMT = "%Y-%m-%d %H:%M:%S"


class ColoredFormatter(logging.Formatter):
    def format(self, record: logging.LogRecord) -> str:
        # Add color to levelname
        levelname = record.levelname
        if levelname in COLORS:
            colored_levelname = f"{COLORS[levelname]}{levelname}{RESET}"
            record.levelname = colored_levelname
        else:
            record.levelname = levelname

        return super().format(record)


def _level_from_string(level_str: str) -> int:
    """
    Convert a logging level name (e.g. "INFO") to the numeric level.
    Defaults to logging.INFO for unknown values.
    """
    if not level_str:
        return logging.INFO
    return getattr(logging, level_str.upper(), logging.INFO)


def logger(name: str = "<Application_Name>") -> logging.Logger:
    """
    Factory that returns a configured logger instance with colored output.

    Usage:
        from src.utils.logger import logger
        log = logger(__name__)   # preferred
        # or keep the default name:
        log = logger()
    """
    lvl = _level_from_string(settings.logging_level)

    _logger = logging.getLogger(name)

    if not _logger.handlers:
        _logger.setLevel(lvl)

        handler = logging.StreamHandler(sys.stdout)
        handler.setLevel(lvl)
        handler.setFormatter(ColoredFormatter(fmt=LOG_FORMAT, datefmt=DATEFMT))

        _logger.addHandler(handler)

    return _logger



import pytest
from unittest.mock import patch, AsyncMock
from fastappear.utils.db import _make_async_uri, get_db, init_models, get_sync_engine


def test_make_async_uri_postgresql() -> None:
    uri = "postgresql://user:pass@localhost/db"
    result = _make_async_uri(uri)
    assert result == "postgresql+asyncpg://user:pass@localhost/db"


def test_make_async_uri_already_async() -> None:
    uri = "postgresql+asyncpg://user:pass@localhost/db"
    result = _make_async_uri(uri)
    assert result == uri


def test_make_async_uri_other() -> None:
    uri = "sqlite:///db.sqlite"
    result = _make_async_uri(uri)
    assert result == uri


@pytest.mark.asyncio
async def test_get_db() -> None:
    with patch("src.utils.db.async_session") as mock_async_session:
        mock_session = AsyncMock()
        mock_async_session.return_value = mock_session
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        # Collect all yielded sessions
        sessions = []
        async for session in get_db():
            sessions.append(session)
            break  # Only take one

        assert len(sessions) == 1
        assert sessions[0] == mock_session

        # Check that the context manager was entered
        mock_session.__aenter__.assert_called_once()


@pytest.mark.asyncio
async def test_init_models() -> None:
    with patch("src.utils.db.async_engine") as mock_engine:
        mock_conn = AsyncMock()
        mock_engine.begin.return_value = mock_conn
        mock_conn.__aenter__ = AsyncMock(return_value=mock_conn)
        mock_conn.__aexit__ = AsyncMock(return_value=None)
        mock_conn.run_sync = AsyncMock()

        await init_models(mock_engine)

        mock_conn.run_sync.assert_called_once()


def test_get_sync_engine() -> None:
    with (
        patch("src.utils.db.settings") as mock_settings,
        patch("src.utils.db.create_engine") as mock_create_engine,
    ):

        mock_settings.db_uri = "postgresql+asyncpg://user:pass@localhost/db"
        mock_engine = "mock_engine"
        mock_create_engine.return_value = mock_engine

        engine = get_sync_engine()
        assert engine == mock_engine
        mock_create_engine.assert_called_once()
        args = mock_create_engine.call_args[0]
        assert "postgresql://" in args[0]  # Should convert back



import json
import logging
import multiprocessing
import os

from fastappear.utils.logger import ColoredFormatter, _level_from_string

workers_per_core_str = os.getenv("WORKERS_PER_CORE", "1")
max_workers_str = os.getenv("MAX_WORKERS", "10")
use_max_workers = None

if max_workers_str:
    use_max_workers = int(max_workers_str)

web_concurrency_str = os.getenv("WEB_CONCURRENCY", None)
host = os.getenv("HOST", "0.0.0.0")
port = os.getenv("PORT", "8000")
bind_env = os.getenv("BIND", None)
use_loglevel = os.getenv("LOG_LEVEL", "info")

# Configure Gunicorn logger with colored output
gunicorn_logger = logging.getLogger("gunicorn")
if not gunicorn_logger.handlers:
    lvl = _level_from_string(use_loglevel)
    gunicorn_logger.setLevel(lvl)
    handler = logging.StreamHandler()
    handler.setLevel(lvl)
    handler.setFormatter(
        ColoredFormatter(
            fmt="[GUNICORN] %(levelname)s %(asctime)s - %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )
    )
    gunicorn_logger.addHandler(handler)

# Configure access and error loggers
access_logger = logging.getLogger("gunicorn.access")
if not access_logger.handlers:
    access_logger.setLevel(logging.INFO)
    access_handler = logging.StreamHandler()
    access_handler.setFormatter(
        ColoredFormatter(
            fmt="[ACCESS] %(asctime)s - %(message)s", datefmt="%Y-%m-%d %H:%M:%S"
        )
    )
    access_logger.addHandler(access_handler)

error_logger = logging.getLogger("gunicorn.error")
if not error_logger.handlers:
    error_logger.setLevel(_level_from_string(use_loglevel))
    error_handler = logging.StreamHandler()
    error_handler.setFormatter(
        ColoredFormatter(
            fmt="[ERROR] %(levelname)s %(asctime)s - %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )
    )
    error_logger.addHandler(error_handler)

if bind_env:
    use_bind = bind_env
else:
    use_bind = f"{host}:{port}"

cores = multiprocessing.cpu_count()
workers_per_core = float(workers_per_core_str)
default_web_concurrency = workers_per_core * cores

if web_concurrency_str:
    web_concurrency = int(web_concurrency_str)
    assert web_concurrency > 0
else:
    web_concurrency = max(int(default_web_concurrency), 2)
    if use_max_workers:
        web_concurrency = min(web_concurrency, use_max_workers)

accesslog_var = os.getenv("ACCESS_LOG", "-")
use_accesslog = accesslog_var or None
errorlog_var = os.getenv("ERROR_LOG", "-")
use_errorlog = errorlog_var or None
graceful_timeout_str = os.getenv("GRACEFUL_TIMEOUT", "60")
timeout_str = os.getenv("TIMEOUT", "60")
keepalive_str = os.getenv("KEEP_ALIVE", "5")

# Gunicorn config variables
worker_class = "uvicorn.workers.UvicornWorker"
loglevel = use_loglevel
workers = web_concurrency
bind = use_bind
errorlog = use_errorlog
worker_tmp_dir = "/dev/shm"
accesslog = use_accesslog
graceful_timeout = int(graceful_timeout_str)
timeout = int(timeout_str)
keepalive = int(keepalive_str)

# For debugging and testing
log_data = {
    "loglevel": loglevel,
    "workers": workers,
    "bind": bind,
    "graceful_timeout": graceful_timeout,
    "timeout": timeout,
    "keepalive": keepalive,
    "errorlog": errorlog,
    "accesslog": accesslog,
    # Additional, non-gunicorn variables
    "workers_per_core": workers_per_core,
    "use_max_workers": use_max_workers,
    "host": host,
    "port": port,
}

print(json.dumps(log_data))



import pytest
from unittest.mock import patch
from datetime import datetime, timedelta
import jwt
from fastappear.utils.jwt import (
    create_access_token,
    decode_token,
    revoke_jti,
    BLACKLIST,
    ACCESS_TOKEN_EXPIRE_HOURS,
)


def test_create_access_token() -> None:
    with (
        patch("src.utils.jwt.datetime") as mock_datetime,
        patch("src.utils.jwt._make_jti") as mock_jti,
        patch("src.utils.jwt.JWT_SECRET", "secret"),
        patch("src.utils.jwt.JWT_ALGORITHM", "HS256"),
        patch("src.config.settings") as mock_settings,
    ):

        mock_settings.jwt_access_token_expire_minutes = 60

        # Set to past date to avoid immature signature
        mock_now = datetime(2023, 1, 1, 12, 0, 0)
        mock_datetime.now.return_value = mock_now
        mock_jti.return_value = "test-jti"

        token = create_access_token("test-identity")

        # Decode to verify
        payload = jwt.decode(
            token,
            "secret",
            algorithms=["HS256"],
            options={"verify_iat": False, "verify_exp": False},
        )
        assert payload["sub"] == "test-identity"
        assert payload["jti"] == "test-jti"
        assert payload["iat"] == int(mock_now.timestamp())
        expected_exp = int(
            (mock_now + timedelta(hours=ACCESS_TOKEN_EXPIRE_HOURS)).timestamp()
        )
        assert payload["exp"] == expected_exp


def test_decode_token_valid() -> None:
    with (
        patch("src.utils.jwt.JWT_SECRET", "secret"),
        patch("src.utils.jwt.JWT_ALGORITHM", "HS256"),
    ):

        # Add exp to future
        future = datetime(2026, 1, 1, 12, 0, 0)
        payload = {"sub": "test", "jti": "jti123", "exp": int(future.timestamp())}
        token = jwt.encode(payload, "secret", algorithm="HS256")

        decoded = decode_token(token)
        assert decoded["sub"] == "test"
        assert decoded["jti"] == "jti123"


def test_decode_token_revoked() -> None:
    with (
        patch("src.utils.jwt.JWT_SECRET", "secret"),
        patch("src.utils.jwt.JWT_ALGORITHM", "HS256"),
    ):

        future = datetime(2026, 1, 1, 12, 0, 0)
        payload = {"sub": "test", "jti": "revoked-jti", "exp": int(future.timestamp())}
        token = jwt.encode(payload, "secret", algorithm="HS256")

        BLACKLIST.add("revoked-jti")

        with pytest.raises(jwt.InvalidTokenError):
            decode_token(token)

        BLACKLIST.clear()


def test_decode_token_invalid() -> None:
    with (
        patch("src.utils.jwt.JWT_SECRET", "secret"),
        patch("src.utils.jwt.JWT_ALGORITHM", "HS256"),
    ):

        # Invalid token
        with pytest.raises(jwt.InvalidTokenError):
            decode_token("invalid-token")


def test_revoke_jti() -> None:
    jti = "test-jti"
    assert jti not in BLACKLIST
    revoke_jti(jti)
    assert jti in BLACKLIST
    BLACKLIST.clear()



import os
import sys


def main() -> None:
    if len(sys.argv) != 2:
        print("Usage: python generate_module.py <directory_name>")
        sys.exit(1)

    dir_name = sys.argv[1]

    # Change to src directory
    src_path = os.path.join(os.getcwd(), "src")
    if not os.path.exists(src_path):
        print("src directory does not exist.")
        sys.exit(1)

    os.chdir(src_path)

    # Create the new directory
    new_dir_path = os.path.join(src_path, dir_name)
    os.makedirs(new_dir_path, exist_ok=True)

    # Create __init__.py in the new directory
    with open(os.path.join(new_dir_path, "__init__.py"), "w") as f:
        f.write("")

    # Create schema.py
    with open(os.path.join(new_dir_path, "schema.py"), "w") as f:
        f.write("")

    # Create model.py
    with open(os.path.join(new_dir_path, "model.py"), "w") as f:
        f.write("")

    # Create routes directory and __init__.py
    routes_dir = os.path.join(new_dir_path, "routes")
    os.makedirs(routes_dir, exist_ok=True)
    with open(os.path.join(routes_dir, "__init__.py"), "w") as f:
        f.write("")

    # Create services directory and __init__.py
    services_dir = os.path.join(new_dir_path, "services")
    os.makedirs(services_dir, exist_ok=True)
    with open(os.path.join(services_dir, "__init__.py"), "w") as f:
        f.write("")

    print(f"Created directory {dir_name} in src with required files.")


if __name__ == "__main__":
    main()



import time
from typing import Any, Callable, TypeVar, Dict, AsyncGenerator
from contextlib import asynccontextmanager
import uvicorn
from fastapi import FastAPI, Request, Response, APIRouter
from fastapi.middleware.cors import CORSMiddleware
from fastappear.config import settings
from fastappear.utils.db import init_models, async_session, async_engine
from sqlalchemy.ext.asyncio import AsyncSession
from fastappear.utils.logger import logger
import argparse
import shutil
import os

description = """
<Application_Name> API's
"""

log = logger()


async def create_default_admin_if_missing(session: AsyncSession) -> None:
    """No-op helper to create a default admin if it does not exist.

    This is intentionally lightweight — it logs the operation. Replace with your
    production logic (creating a seeded admin user) in a real application.
    """
    log.debug("checked for default admin (no-op)")
    return None


@asynccontextmanager
async def lifespan(app: FastAPI) -> AsyncGenerator[None, None]:
    """Application lifespan context.

    - initializes the DB models at startup
    - tries to create a default admin if missing (no-op in minimal setup)
    - disposes the async engine at shutdown
    """
    log.info("Starting <Application_Name> API...")
    await init_models()
    # create_default_admin_if_missing is optional; implement as a no-op if not available
    try:
        async with async_session() as session:
            await create_default_admin_if_missing(session)
    except NameError:
        log.debug("create_default_admin_if_missing not available, skipping")
    except Exception:
        log.exception("Failed to create default admin, continuing startup")

    log.info("Startup complete.")

    yield

    log.info("Shutting down <Application_Name> API...")
    # Properly dispose the underlying engine when shutting down
    try:
        await async_engine.dispose()
    except Exception:
        log.exception("Error on engine dispose")
    log.info("Shutdown complete.")


app = FastAPI(
    title="<Application_Name>",
    description=description,
    version="1.0.0",
    docs_url="/docs",
    redoc_url="/redoc",
    root_path=settings.root_path,
    lifespan=lifespan,
)

app.add_middleware(
    CORSMiddleware,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
    allow_origins=["*"],
)


@app.get("/", tags=["Health"])
async def health_check() -> Dict[str, str]:
    return {"status": "ok", "message": "<Application_Name> API is running"}


F = TypeVar("F", bound=Callable[..., Any])


@app.middleware("http")
async def process_time_log_middleware(
    request: Request, call_next: Callable[[Request], Any]
) -> Response:
    start_time = time.time()
    response: Response = await call_next(request)
    process_time = str(round(time.time() - start_time, 3))
    response.headers["X-Process-Time"] = process_time
    log.info(
        "Method=%s Path=%s StatusCode=%s ProcessTime=%s",
        request.method,
        request.url.path,
        response.status_code,
        process_time,
    )
    return response


## Minimal routers for demonstration
login_router = APIRouter(prefix="/auth", tags=["auth"])
register_router = APIRouter(prefix="/auth", tags=["auth"])


@login_router.post("/login")
async def _login() -> Dict[str, str]:
    return {"detail": "login endpoint (stub)"}


@register_router.post("/register")
async def _register() -> Dict[str, str]:
    return {"detail": "register endpoint (stub)"}


app.include_router(login_router)
app.include_router(register_router)


def main() -> None:
    parser = argparse.ArgumentParser(description="FastAppear - FastAPI application")
    parser.add_argument("--init", action="store_true", help="Initialize a new FastAppear project in the current directory")
    args = parser.parse_args()

    if args.init:
        init_project()
    else:
        run_server()


def init_project() -> None:
    """Initialize a new FastAppear project by copying template files."""
    template_dir = os.path.join(os.path.dirname(__file__), "template")
    if not os.path.exists(template_dir):
        print("Error: Template directory not found in package.")
        return

    current_dir = os.getcwd()
    for item in os.listdir(template_dir):
        src = os.path.join(template_dir, item)
        dst = os.path.join(current_dir, item)
        if os.path.exists(dst):
            print(f"Skipping {item}: already exists")
        else:
            if os.path.isdir(src):
                shutil.copytree(src, dst)
            else:
                shutil.copy2(src, dst)
            print(f"Copied {item}")

    print("FastAppear project initialized!")


def run_server() -> None:
    """Run the FastAPI server."""
    uvicorn.run(
        app,
        host="0.0.0.0",
        port=8000,
        log_level="debug",
        reload=True,
    )


if __name__ == "__main__":
    main()



from fastappear.config import settings
from datetime import datetime, timedelta
from typing import Dict, Any
import jwt


JWT_SECRET = settings.jwt_secret_key
JWT_ALGORITHM = settings.jwt_algorithm
ACCESS_TOKEN_EXPIRE_HOURS = settings.jwt_access_token_expire_minutes // 60

BLACKLIST = set()


def _make_jti() -> str:
    import uuid

    return str(uuid.uuid4())


def create_access_token(identity: str) -> str:
    """
    Create a JWT with 'sub' and 'jti' claims and expiry.
    Returns a string token.
    """
    now = datetime.now()
    exp = now + timedelta(hours=ACCESS_TOKEN_EXPIRE_HOURS)
    jti = _make_jti()
    payload = {
        "sub": str(identity),
        "iat": int(now.timestamp()),
        "exp": int(exp.timestamp()),
        "jti": jti,
    }
    token = jwt.encode(payload, JWT_SECRET, algorithm=JWT_ALGORITHM)
    return token


def decode_token(token: str) -> Dict[str, Any]:
    """
    Decode and verify token. Raises jwt exceptions on invalid/expired token.
    Also checks blacklist.
    """
    payload = jwt.decode(token, JWT_SECRET, algorithms=[JWT_ALGORITHM])
    jti = payload.get("jti")
    if jti in BLACKLIST:
        raise jwt.InvalidTokenError("Token revoked")
    return payload


def revoke_jti(jti: str) -> None:
    BLACKLIST.add(jti)



import logging
from unittest.mock import patch
from fastappear.utils.logger import logger, _level_from_string, ColoredFormatter, COLORS, RESET


def test_level_from_string() -> None:
    assert _level_from_string("DEBUG") == logging.DEBUG
    assert _level_from_string("INFO") == logging.INFO
    assert _level_from_string("WARNING") == logging.WARNING
    assert _level_from_string("ERROR") == logging.ERROR
    assert _level_from_string("CRITICAL") == logging.CRITICAL
    assert _level_from_string("invalid") == logging.INFO  # default
    assert _level_from_string("") == logging.INFO  # default


def test_colored_formatter() -> None:
    formatter = ColoredFormatter(fmt="%(levelname)s: %(message)s")

    # Create a mock record
    record = logging.LogRecord(
        name="test",
        level=logging.INFO,
        pathname="",
        lineno=0,
        msg="Test message",
        args=(),
        exc_info=None,
    )

    formatted = formatter.format(record)
    assert "Test message" in formatted
    # Should contain color codes
    assert "\033[" in formatted
    assert RESET in formatted


def test_logger_creation() -> None:
    with (
        patch("src.utils.logger._level_from_string") as mock_level,
        patch("src.config.settings") as mock_settings,
    ):
        mock_level.return_value = logging.DEBUG
        mock_settings.logging_level = "DEBUG"

        test_logger = logger("test-logger")
        assert test_logger.name == "test-logger"
        assert test_logger.level == logging.DEBUG
        assert len(test_logger.handlers) == 1

        handler = test_logger.handlers[0]
        assert isinstance(handler, logging.StreamHandler)
        assert isinstance(handler.formatter, ColoredFormatter)


def test_logger_default_name() -> None:
    with patch("src.config.settings") as mock_settings:
        mock_settings.logging_level = "INFO"

        default_logger = logger()
        assert default_logger.name == "<Application_Name>"


def test_colors_dict() -> None:
    assert "DEBUG" in COLORS
    assert "INFO" in COLORS
    assert "ERROR" in COLORS
    assert isinstance(COLORS["INFO"], str)
    assert COLORS["INFO"].startswith("\033[")



from pathlib import Path
from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict

BASE_DIR = Path(__file__).resolve().parent
ENV_PATH = BASE_DIR.parent / ".env"


class Settings(BaseSettings):
    model_config = SettingsConfigDict(
        env_file=ENV_PATH, env_file_encoding="utf-8", extra="ignore"
    )

    db_uri: str = Field(..., env="DB_URI")
    root_path: str = Field("", env="ROOT_PATH")
    logging_level: str = Field("INFO", env="LOGGING_LEVEL")
    jwt_secret_key: str = Field(..., env="JWT_SECRET_KEY")
    jwt_algorithm: str = Field("HS512", env="JWT_ALGORITHM")
    jwt_access_token_expire_minutes: int = Field(
        720, env="JWT_ACCESS_TOKEN_EXPIRE_MINUTES"
    )
    jwt_refresh_token_expire_days: int = Field(7, env="JWT_REFRESH_TOKEN_EXPIRE_DAYS")


settings = Settings()
